import Flutter
import UIKit
import HailifyFleetSDK
import CoreLocation
public class SwiftHailifyPlugin: NSObject, FlutterPlugin, FleetSDKDelegate {
  public static func register(with registrar: FlutterPluginRegistrar) {
    let channel = FlutterMethodChannel(name: "hailify", binaryMessenger: registrar.messenger())
    let instance = SwiftHailifyPlugin()
    registrar.addMethodCallDelegate(instance, channel: channel)
  }
  public func handle(_ call: FlutterMethodCall, result: @escaping FlutterResult) {
        switch call.method {
            case MethodChannelNames.INITIALIZE:
                self.initialize(call, result: result)
            case MethodChannelNames.SET_DRIVER_ID:
                self.setDriverId(call, result: result)
            case MethodChannelNames.ENABLE_SANDBOX:
                self.enableSandbox(call, result: result)
            case MethodChannelNames.USE_SDK_LOCATION_ENGINE:
                self.useSDKLocationEngine(call, result: result)
            case MethodChannelNames.SET_LOCATION:
                self.setLocation(call, result: result)
            case MethodChannelNames.CANCEL_ORDER:
                self.cancelOrder(call, result: result)
            case MethodChannelNames.INITIALIZE_DELIVERY_VIEW:
                self.initializeDeliveryView()
            case MethodChannelNames.SET_CALL_LISTENER:
                self.setCallListener()
            case MethodChannelNames.SET_NAVIGATION_LISTENER:
                self.setNavigationListener()
            case MethodChannelNames.SET_ORDER_STATUS_LISTENER:
                self.setOrderStatusListener()
            case MethodChannelNames.SET_COMPLETE_LISTENER:
                self.setCompleteListener()
            case MethodChannelNames.SHOW_ORDER_POPUP:
                self.showOrderPopup()
            case MethodChannelNames.GET_ORDER_STATUS:
                self.getCurrentOrderStatus(result)
            case MethodChannelNames.SET_MAX_HEIGHT_PERCENTAGE:
                self.setMaxHeightPercentage(call, result: result)
            default:
                result(FlutterMethodNotImplemented)
        }
    }
    public func initialize (_ call: FlutterMethodCall, result: @escaping FlutterResult)  {
        if let dicParam = call.arguments as? Dictionary<String, Any> {
            //let fleetName = dicParam["fleetName"] ?? ""
            if let token = dicParam["token"] as? String {
                FleetSDK.shared.initilize(withKey: token)
            }
            result(nil)
        }
    }
    public func setDriverId(_ call: FlutterMethodCall, result: @escaping FlutterResult) {
        if let dicParam = call.arguments as? Dictionary<String, Any> {
            if let driverId = dicParam["driverId"] as? String {
                FleetSDK.shared.setDriverID(driverId)
            }
            result(nil)
        }
    }
    public func enableSandbox(_ call: FlutterMethodCall, result: @escaping FlutterResult) {
        if let dicParam = call.arguments as? Dictionary<String, Any> {
            if let isEnableSandbox = dicParam["isEnableSandbox"] as? Bool {
                FleetSDK.shared.enableSandbox = isEnableSandbox
            }
            result(nil)
        }
    }
    public func useSDKLocationEngine(_ call: FlutterMethodCall, result: @escaping FlutterResult) {
        if let dicParam = call.arguments as? Dictionary<String, Any> {
            if let useSDKLocationEngine = dicParam["useSDKLocationEngine"] as? Bool {
                FleetSDK.shared.useSDKLocationEngine = useSDKLocationEngine
            }
            result(nil)
        }
    }
    public func setLocation(_ call: FlutterMethodCall, result: @escaping FlutterResult) {
        if let dicParam = call.arguments as? Dictionary<String, Any> {
            var lat : Double = 0
            var lng : Double = 0
            if let latitude = dicParam["latitude"] as? Double {
                lat = latitude
            }
            else if let latitude = dicParam["latitude"] as? Float {
                lat = Double(latitude)
            }
            if let longitude = dicParam["longitude"] as? Double {
                lng = longitude
            }
            else if let longitude = dicParam["longitude"] as? Float {
                lng = Double(longitude)
            }
            let location = CLLocation(latitude: lat, longitude: lng)
            FleetSDK.shared.setLocation(location: location)
            result(nil)
        }
    }
    public func cancelOrder(_ call: FlutterMethodCall, result: @escaping FlutterResult) {
        if let dicParam = call.arguments as? Dictionary<String, Any> {
            if let additionalResponse = dicParam["additionalResponse"] as? String {
                FleetSDK.shared.cancelOrder(aditionalResponse: additionalResponse)
            }
            result(nil)
        }
    }
    public func setMaxHeightPercentage(_ call: FlutterMethodCall, result: @escaping FlutterResult) {
        if let dicParam = call.arguments as? Dictionary<String, Any> {
            if let height = dicParam["height"] as? Int {
                FleetSDK.shared.maxHeightPercentage = UInt(height)
            }
            else if let height = dicParam["height"] as? UInt {
                FleetSDK.shared.maxHeightPercentage = height
            }
            else if let height = dicParam["height"] as? Float {
                FleetSDK.shared.maxHeightPercentage = UInt(height)
            }
            else {
                FleetSDK.shared.maxHeightPercentage = 100
            }
            result(nil)
        }
    }
    public func getCurrentOrderStatus(_ result: @escaping FlutterResult) {
        let orderCurrentStatus : String = FleetSDK.shared.getOrderCurrentStatus()
        result(orderCurrentStatus)
    }
    public func initializeDeliveryView() {
        FleetSDK.shared.delegate = self
        if let viewCtr = UIApplication.shared.delegate?.window??.rootViewController {
            FleetSDK.shared.showOrderPopup(viewController: viewCtr)
        }
        DispatchQueue.main.async {
            if let channel = self.returnChannel() {
                channel.invokeMethod(MethodChannelNames.INITIALIZE_DELIVERY_VIEW_CALLBACK, arguments: nil)
            }
        }
    }
    public func showOrderPopup() {
        if let viewCtr = UIApplication.shared.delegate?.window??.rootViewController {
            FleetSDK.shared.showOrderPopup(viewController: viewCtr)
        }
    }
    // MARK: - define callback IDs
    var strCallListnerId : String?
    var strNavigationListnerId : String?
    var strOrderStatusListnerId : String?
    var strBatchCompleteListnerId : String?
    public func setCallListener() {
        self.strCallListnerId = "setCallListener"
    }
    public func setNavigationListener() {
        self.strNavigationListnerId = "setNavigationListener"
    }
    public func setOrderStatusListener() {
        self.strOrderStatusListnerId = "setOrderStatusListener"
    }
    public func setCompleteListener() {
        self.strBatchCompleteListnerId = "setCompleteListener"
    }
    public func returnChannel () -> FlutterMethodChannel? {
        if let controller = UIApplication.shared.delegate?.window??.rootViewController as? FlutterViewController {
            let channel = FlutterMethodChannel(name: "hailify", binaryMessenger: controller.binaryMessenger)
            return channel
        }
        return nil
    }
    // MARK: - FleetSDK Delegate
    public func navigateToLocation(latitude: Double, longitude: Double) {
        DispatchQueue.main.async {
            if self.strNavigationListnerId != nil {
                if let channel = self.returnChannel() {
                channel.invokeMethod(MethodChannelNames.SET_NAVIGATION_LISTENER_CALLBACK, arguments: ["latitude" : latitude, "longitude" : longitude])
                }
            }
            else {
              GlobalMethods.navigateToMapApp(latitude: latitude, longitude: longitude)
            }
        }
    }
    public func callToLocation(phoneNumber: String) {
        DispatchQueue.main.async {
            if self.strCallListnerId != nil {
                if let channel = self.returnChannel() {
                    channel.invokeMethod(MethodChannelNames.SET_CALL_LISTENER_CALLBACK, arguments: ["phoneNumber" : phoneNumber])
                }
            }
            else {
                phoneNumber.makeAPhoneCall()
            }
        }
    }
    public func updatedOrderStatus(deliveryID: String, orderStatus: String) {

        DispatchQueue.main.async {
            if self.strOrderStatusListnerId != nil {
                if let channel = self.returnChannel() {
                    channel.invokeMethod(MethodChannelNames.SET_ORDER_STATUS_LISTENER_CALLBACK, arguments: ["orderId": deliveryID, "orderStatus": orderStatus])
                }
            }
        }
    }
    public func batchOrderCompleted() {

        DispatchQueue.main.async {
            if self.strBatchCompleteListnerId != nil {
                if let channel = self.returnChannel() {
                    channel.invokeMethod(MethodChannelNames.SET_COMPLETE_LISTENER_CALLBACK, arguments: nil)
                }
            }
        }
    }
}
enum MethodChannelNames {
    static let INITIALIZE = "initialize"
    static let SET_DRIVER_ID = "setDriverId"
    static let ENABLE_SANDBOX = "enableSandbox"
    static let USE_SDK_LOCATION_ENGINE = "useSDKLocationEngine"
    static let SET_LOCATION = "setLocation"
    static let CANCEL_ORDER = "cancelOrder"
    static let INITIALIZE_DELIVERY_VIEW = "initializeDeliveryView"
    static let INITIALIZE_DELIVERY_VIEW_CALLBACK = "initializeDeliveryViewCallback"
    static let SET_CALL_LISTENER = "setCallListener"
    static let SET_CALL_LISTENER_CALLBACK = "setCallListenerCallback"
    static let SET_NAVIGATION_LISTENER = "setNavigationListener"
    static let SET_NAVIGATION_LISTENER_CALLBACK = "setNavigationListenerCallback"
    static let SET_ORDER_STATUS_LISTENER = "setOrderStatusListener"
    static let SET_ORDER_STATUS_LISTENER_CALLBACK = "setOrderStatusListenerCallback"
    static let SET_COMPLETE_LISTENER = "setCompleteListener"
    static let SET_COMPLETE_LISTENER_CALLBACK = "setCompleteListenerCallback"
    static let SHOW_ORDER_POPUP = "showOrderPopup"
    static let GET_ORDER_STATUS = "getCurrentOrderStatus"
    static let SET_MAX_HEIGHT_PERCENTAGE = "setMaxHeightPercentage"
}