import 'dart:async';

import 'package:flutter/services.dart';
import 'package:hailify/constants/method_channels.dart';

class Hailify {
  static const MethodChannel _channel = MethodChannel('hailify');

  static late final Function? _initializeDeliveryViewCallback;

  static late final Function(String phoneNumber)? _callListenerCallback;

  static late final Function(double latitude, double longitude)?
  _navigationListenerCallback;

  static late final Function(String orderStatus, String orderId)?
  _orderStatusListenerCallback;

  static late final Function? _completeListenerCallback;

  static void _setMethodChannelListner() {
    _channel.setMethodCallHandler(_processEngineOutput);
  }

  static Future<void> _processEngineOutput(MethodCall call) async {
    switch (call.method) {
      case MethodChannelNames.initializeDeliveryViewCallback:
        _initializeDeliveryViewCallback?.call();
        break;
      case MethodChannelNames.setCallListenerCallback:
        String phoneNumber = call.arguments["phoneNumber"];
        _callListenerCallback?.call(phoneNumber);
        break;
      case MethodChannelNames.setNavigationListenerCallback:
        double latitude = call.arguments["latitude"];
        double longitude = call.arguments["longitude"];
        _navigationListenerCallback?.call(latitude, longitude);
        break;
      case MethodChannelNames.setOrderStatusListenerCallback:
        String status = call.arguments["orderStatus"];
        String orderId = call.arguments["orderId"];
        _orderStatusListenerCallback?.call(status, orderId);
        break;
      case MethodChannelNames.setCompleteListenerCallback:
        _completeListenerCallback?.call();
        break;
      default:
        break;
    }
  }

  static void initilize(String fleetName, String fleetToken) {
    _setMethodChannelListner();
    _channel.invokeMethod(MethodChannelNames.initialize,
        {"fleetName": fleetName, "token": fleetToken});
  }

  static void setDriverId(String driverId) {
    _channel
        .invokeMethod(MethodChannelNames.setDriverId, {"driverId": driverId});
  }

  static void enableSandbox(bool isEnableSandbox) {
    _channel.invokeMethod(MethodChannelNames.enableSandbox,
        {"isEnableSandbox": isEnableSandbox});
  }

  static void useSDKLocationEngine(bool useSDKLocationEngine) {
    _channel.invokeMethod(MethodChannelNames.useSDKLocationEngine,
        {"useSDKLocationEngine": useSDKLocationEngine});
  }

  static void setLocation(double latitude, double longitude, double accuracy) {
    _channel.invokeMethod(MethodChannelNames.setLocation,
        {"latitude": latitude, "longitude": longitude, "accuracy": accuracy});
  }

  static void setMaxHeightPercentage(int height) {
    _channel.invokeMethod(
        MethodChannelNames.setMaxHeightPercentage, {"height": height});
  }

  static void initializeDeliveryView(Function callback) {
    _initializeDeliveryViewCallback = callback;
    _channel.invokeMethod(MethodChannelNames.initializeDeliveryView);
  }

  static void setCallListener(Function(String number) callback) {
    _callListenerCallback = callback;
    _channel.invokeMethod(MethodChannelNames.setCallListener);
  }

  static void setNavigationListener(
      Function(double latitude, double longitude) callback) {
    _navigationListenerCallback = callback;
    _channel.invokeMethod(MethodChannelNames.setNavigationListener);
  }

  static void setOrderStatusListener(
      Function(String orderStatus, String orderId) callback) {
    _orderStatusListenerCallback = callback;
    _channel.invokeMethod(MethodChannelNames.setOrderStatusListener);
  }

  static void setCompleteListener(Function callback) {
    _completeListenerCallback = callback;
    _channel.invokeMethod(MethodChannelNames.setCompleteListener);
  }

  static void showOrderPopup() {
    _channel.invokeMethod(MethodChannelNames.showOrderPopup);
  }

  static Future<String?> get getCurrentOrderStatus async {
    String? result;
    result = await _channel
        .invokeMethod<String>(MethodChannelNames.getCurrentOrderStatus);
    return result;
  }
}